import { getAllResources } from "../../db/resources";
import { updateSystemResources } from "../../db/systems";
import SystemManager from "./SystemManager";
import ResourceManager from "./abstract/ResourceManager";

export default class SystemResourceManager extends ResourceManager {
    system: SystemManager;

    constructor(system: SystemManager) {
        super();
        this.system = system;
    }

    async init(resourceData?: { id: string, amount: number }[]) {
        const resources = await getAllResources();

        this.resourcesDB = resources;

        if(!resourceData || resourceData.length === 0) {
            resourceData = [
                {
                    id: "coal",
                    amount: 11
                },
                {
                    id: "iron",
                    amount: 22
                },
                {
                    id: "gold",
                    amount: 33
                },
                {
                    id: "water",
                    amount: 44
                },
                {
                    id: "sulfuric-acid",
                    amount: 55
                },
                {
                    id: "liquid-nitrogen",
                    amount: 66
                },
                {
                    id: "hydrogen",
                    amount: 77
                },
                {
                    id: "oxygen",
                    amount: 88
                },
                {
                    id: "helium-3",
                    amount: 99
                }
            ];

            await updateSystemResources(this.system.data._id, this.resources);
        }
    
        resourceData.forEach(resource => {
            const resFromDB = resources.find(res => res.id === resource.id);
            if(resFromDB) this.resources.push({
                id: resource.id,
                amount: resource.amount,
                lastUpdated: null,
                data: resFromDB
            })
        });

        return this;
    }

    getStorageCapacities(): { id: string; capacity: number; }[] {
        const output: { id: string, capacity: number }[] = [
            {
                id: "coal",
                capacity: 10_000
            }, {
                id: "iron",
                capacity: 10_000
            }, {
                id: "gold",
                capacity: 10_000
            }, {
                id: "water",
                capacity: 10_000
            }, {
                id: "sulfuric-acid",
                capacity: 10_000
            }, {
                id: "liquid-nitrogen",
                capacity: 10_000
            }, {
                id: "hydrogen",
                capacity: 10_000
            }, {
                id: "oxygen",
                capacity: 10_000
            }, {
                id: "helium-3",
                capacity: 10_000
            }, {
                id: "uranium",
                capacity: 10_000
            }, {
                id: "oil",
                capacity: 10_000
            }, {
                id: "diesel",
                capacity: 10_000
            }, {
                id: "kerosene",
                capacity: 10_000
            }, {
                id: "deuterium",
                capacity: 10_000
            }, {
                id: "tritium",
                capacity: 10_000
            }
        ];

        const storageStation = this.system.structures.structures.find(s => s.data.id === 'storage-station');

        if(typeof storageStation !== 'undefined') {
            output.forEach(res => {
                res.capacity += storageStation.level * 40_000;
            });
        }

        return output;
    }

    getPerHourMiningRate() { return []; }

    async sync() {
        await updateSystemResources(this.system.data._id, this.resources.map(res => {
            return {
                id: res.id,
                amount: res.amount,
                lastUpdated: res.lastUpdated,
            }
        }));
    }
}